
# Copyright 2021 by Mobile AI workshop. All Rights Reserved.
#
# IMPORTANT NOTE:
#
# Since the min distance to the object is equal to 0.4m = 400mm, we can replace all unknown values (marked by 0s) with ones.
# In this case, there won't be any problems when computing logarithms in the formulas below (since log[x >= 1] >= 0).

#!/usr/bin/env python
import sys
import os
import os.path
import random
import numpy as np
np.random.seed(42)
from PIL import Image
import scipy.misc
#from skimage.measure import structural_similarity as ssim
from myssim import compare_ssim as ssim


def output_psnr_mse(img_orig, img_out):
    squared_error = np.square(img_orig - img_out)
    mse = np.mean(squared_error)
    psnr = 10 * np.log10(1.0 / mse)
    return psnr

def _open_img(img_p):
    F = scipy.misc.fromimage(Image.open(img_p)).astype(float)/255.0
    return F[:,:,:3]

def _open_img_ssim(img_p):
    F = scipy.misc.fromimage(Image.open(img_p))#.astype(float)
    return F#[:,:,:3]


def compute_psnr(ref_im, res_im):
    return output_psnr_mse(
        _open_img(os.path.join(input_dir,'ref',ref_im)),
        _open_img(os.path.join(input_dir,'res',res_im))
        )

#def compute_mssim(ref_im, res_im):
#    ref_img = _open_img_ssim(os.path.join(input_dir,'ref',ref_im))
#    res_img = _open_img_ssim(os.path.join(input_dir,'res',res_im))
#    channels = []
#    for i in range(3):
#        channels.append(ssim(ref_img[:,:,i],res_img[:,:,i],
#        gaussian_weights=True, use_sample_covariance=False))
#    return np.mean(channels)


def compute_rmse(img, target):

    img[img < 1] = 1
    target[target < 1] = 1
    mask = np.asarray(target > 1, dtype=int)

    diff = (target - img) * mask / 1000.0
    num_pixels = float(np.sum(mask > 0))

    return np.sqrt(np.sum(np.square(diff)) / num_pixels)


def compute_si_rmse(img, target):

    img[img < 1] = 1
    target[target < 1] = 1
    mask = np.asarray(target > 1, dtype=int)

    log_diff = (np.log(img) - np.log(target)) * mask
    num_pixels = float(np.sum(mask > 0))

    return np.sqrt(np.sum(np.square(log_diff)) / num_pixels - np.square(np.sum(log_diff)) / np.square(num_pixels))


def compute_avg_log10(img, target):

    img[img < 1] = 1
    target[target < 1] = 1
    mask = np.asarray(target > 1, dtype=int)

    log_diff = (np.log10(img) - np.log10(target)) * mask
    num_pixels = float(np.sum(mask > 0))

    return np.sum(np.absolute(log_diff)) / num_pixels


def compute_rel(img, target):

    img[img < 1] = 1
    target[target < 1] = 1
    mask = np.asarray(target > 1, dtype=int)

    diff = (img - target) * mask
    num_pixels = float(np.sum(mask > 0))

    return np.sum(np.absolute(diff) / target) / num_pixels




# as per the metadata file, input and output directories are the arguments
[_, input_dir, output_dir] = sys.argv

res_dir = os.path.join(input_dir, 'res/')
ref_dir = os.path.join(input_dir, 'ref/')
#print("REF DIR")
#print(ref_dir)


runtime = -1
cpu = -1
data = -1
other = ""
readme_fnames = [p for p in os.listdir(res_dir) if p.lower().startswith('readme')]
try:
    readme_fname = readme_fnames[0]
    print("Parsing extra information from %s"%readme_fname)
    with open(os.path.join(input_dir, 'res', readme_fname)) as readme_file:
        readme = readme_file.readlines()
        lines = [l.strip() for l in readme if l.find(":")>=0]
        runtime = float(":".join(lines[0].split(":")[1:]))
        cpu = int(":".join(lines[1].split(":")[1:]))
        data = int(":".join(lines[2].split(":")[1:]))
        other = ":".join(lines[3].split(":")[1:])
except:
    print("Error occured while parsing readme.txt")
    print("Please make sure you have a line for runtime, cpu/gpu, extra data and other (4 lines in total).")
print("Parsed information:")
print("Runtime: %f"%runtime)
print("CPU/GPU: %d"%cpu)
print("Data: %d"%data)
print("Other: %s"%other)





ref_pngs = sorted([p for p in os.listdir(ref_dir) if p.lower().endswith('png')])
res_pngs = sorted([p for p in os.listdir(res_dir) if p.lower().endswith('png')])
if not (len(ref_pngs)==len(res_pngs)):
    raise Exception('Expected %d .png images'%len(ref_pngs))


scores_rmse = []
for (ref_im, res_im) in zip(ref_pngs, res_pngs):
    print(ref_im,res_im)
    ref_img = _open_img_ssim(os.path.join(input_dir,'ref',ref_im))
    res_img = _open_img_ssim(os.path.join(input_dir,'res',res_im))
    scores_rmse.append(
        compute_rmse(res_img,ref_img)
    )
    #print(scores[-1])
rmse = np.mean(scores_rmse)

scores_si_rmse = []
for (ref_im, res_im) in zip(ref_pngs, res_pngs):
    print(ref_im,res_im)
    ref_img = _open_img_ssim(os.path.join(input_dir,'ref',ref_im))
    res_img = _open_img_ssim(os.path.join(input_dir,'res',res_im))
    scores_si_rmse.append(
        compute_si_rmse(res_img,ref_img)
    )
    #print(scores[-1])
si_rmse = np.mean(scores_si_rmse)


scores_avg_log10 = []
for (ref_im, res_im) in zip(ref_pngs, res_pngs):
    print(ref_im,res_im)
    ref_img = _open_img_ssim(os.path.join(input_dir,'ref',ref_im))
    res_img = _open_img_ssim(os.path.join(input_dir,'res',res_im))
    scores_avg_log10.append(
        compute_avg_log10(res_img,ref_img)
    )
    #print(scores[-1])
avg_log10 = np.mean(scores_avg_log10)

scores_rel = []
for (ref_im, res_im) in zip(ref_pngs, res_pngs):
    print(ref_im,res_im)
    ref_img = _open_img_ssim(os.path.join(input_dir,'ref',ref_im))
    res_img = _open_img_ssim(os.path.join(input_dir,'res',res_im))
    scores_rel.append(
        compute_rel(res_img,ref_img)
    )
    #print(scores[-1])
rel = np.mean(scores_rel)


#scores = []
#for (ref_im, res_im) in zip(ref_pngs, res_pngs):
#    print(ref_im,res_im)
#    scores.append(
#        compute_psnr(ref_im,res_im)
#    )
#    #print(scores[-1])
#psnr = np.mean(scores)


#scores_ssim = []
#for (ref_im, res_im) in zip(ref_pngs, res_pngs):
#    print(ref_im,res_im)
#    scores_ssim.append(
#        compute_mssim(ref_im,res_im)
#        )
#    #print(scores_ssim[-1])
#mssim = np.mean(scores_ssim)



# the scores for the leaderboard must be in a file named "scores.txt"
# https://github.com/codalab/codalab-competitions/wiki/User_Building-a-Scoring-Program-for-a-Competition#directory-structure-for-submissions
with open(os.path.join(output_dir, 'scores.txt'), 'w') as output_file:
    output_file.write("Score1:%f\n"%si_rmse)
    output_file.write("Score2:%f\n"%rmse)
    output_file.write("ExtraRuntime:%f\n"%runtime)
    output_file.write("ExtraPlatform:%d\n"%cpu)
    output_file.write("ExtraData:%d\n"%data)
